#######################################################################
#
# ncm-globuscfg for managing Globus configuration
#
# Novk Judit (judit.novak@cern.ch)
#
# For license conditions see http://www.eu-datagrid.org/license.html
#
# Novak Judit <judit.novak@cern.ch>
# Version 0.0.12 : 05/12/04 18:21
#
# ** Generated file : do not edit **
#
#######################################################################

package NCM::Component::globuscfg;

use strict;
use NCM::Component;
use vars qw(@ISA $EC);
@ISA = qw(NCM::Component);
$EC=LC::Exception::Context->new->will_store_all;
use NCM::Check;

use EDG::WP4::CCM::Element;

use File::Copy;
use LC::Check;

##########################################################################
sub Configure($$@) {
##########################################################################

	my ($self, $config) = @_;
	
	# This location should not be changed!  All of the Globus configuration
	# relies on having this file in this place.
	my $sysconfglobus = "/etc/sysconfig/globus";
	
	# The init.d area to be used.
	my $sbinserv = "/sbin/service";
	
	# global conf. vars 
	my %global;
	my $gge = $config->getElement('/software/components/globuscfg/');
	while ( $gge->hasNextElement() ) {
		my $element = $gge->getNextElement();
		if ( $element->isProperty() ) {
			my $name = $element->getName();
			my $value = $element->getValue();
			$global{$name} = $value;
		}
	}
	
	# Check the necessary values (location and configuration file name).
	unless ( $global{'GLOBUS_LOCATION'} ) { 
		$self->error( "Couldn't determine globus location" ) ; 
		return;
	}
	unless ( $global{'GLOBUS_CONFIG'} ) { 
		$self->error( "Couldn't determine globus configuration file" ) ; 
		return;
	}
	
	unless ( $global{'globus_flavor_name'} ) {
		$self->error( "Couldn't determine globus_flavor_name" ) ; 
		return;
	}
	
	# Generating the sysconfig file

	my $contents = "";
	
	$contents .= "GLOBUS_LOCATION=$global{'GLOBUS_LOCATION'}\n";
	$contents .= "GLOBUS_CONFIG=$global{'GLOBUS_CONFIG'}\n";

	$contents .= "GLOBUS_TCP_PORT_RANGE=$global{'GLOBUS_TCP_PORT_RANGE'}\n"
							if $global{'GLOBUS_TCP_PORT_RANGE'};
	$contents .= "GLOBUS_UDP_PORT_RANGE=$global{'GLOBUS_UDP_PORT_RANGE'}\n"
							if $global{'GLOBUS_UDP_PORT_RANGE'};
	
		
	if ( ! -e $sysconfglobus ) {

	  	open ( SYSCONF, ">".$sysconfglobus  );
		print SYSCONF $contents;
		close ( SYSCONF );
		$self->log("System configuration file $sysconfglobus created\n");
	} else {
		my $result = LC::Check::file( $sysconfglobus,
				backup => ".old",
				contents => $contents
			);
		if ( $result ) {
			$self->log("System configuration file $sysconfglobus updated\n");
		}	
	}


	# Globus config file 

	# mds
	my %mds;
	if ( $config->elementExists('/software/components/globuscfg/mds/') ) {
		my $mdse = $config->getElement('/software/components/globuscfg/mds/');
		while ( $mdse->hasNextElement() ) {
			my $element = $mdse->getNextElement();
			my $name = $element->getName();
			my $value = $element->getValue();
			$mds{$name} = $value;
		 } 
	} 
	
	# mds/gris
	my %gris;	
	if ( $config->elementExists('/software/components/globuscfg/mds/gris/suffix') ) {
		my $mge = $config->getElement('/software/components/globuscfg/mds/gris/suffix');
		$gris{'suffix'} = $mge->getValue();
	}	

	if ( $config->elementExists('/software/components/globuscfg/mds/gris/provider') ) {
		my $mpe = $config->getElement('/software/components/globuscfg/mds/gris/provider');
	    	while( $mpe->hasNextElement() ) {
			my $element = $mpe->getNextElement();
			my $name = $element->getName();
			my $value = $element->getValue();
			$gris{'provider'}{$name} = $value;
		}
	}

	if ($config->elementExists('/software/components/globuscfg/mds/gris/registration')) {
		my $mre = 
			$config->getElement('/software/components/globuscfg/mds/gris/registration');
    	while( $mre->hasNextElement() ) {
		my $regelement = $mre->getNextElement();
		my $regno = $regelement->getName();

		# Since it's a record, it must have elements
    		while( $regelement->hasNextElement() ) {	
			my $recentry = $regelement->getNextElement();
			my $name = $recentry->getName();
			my $value = $recentry->getValue();
			$gris{'registration'}{$regno}{$name} = $value;
			}
		}
	}	
	
	
	# This could not be directly taken from the template!!!
	
	my %giisallow;
	if ($config->elementExists('/software/components/globuscfg/mds/giis/allowedregs')) {
		my $mgae = 
			$config->getElement('/software/components/globuscfg/mds/giis/allowedregs');
	    	while( $mgae->hasNextElement() ) {
			my $gaelement = $mgae->getNextElement();
			my $no = $gaelement->getName();
			while ( $gaelement->hasNextElement() ) {
				my $element = $gaelement->getNextElement();
				my $entry = $element->getName();
				if ( $element->isProperty() ) {
					my $value = $element->getValue();
					$giisallow{$no}{$entry} = $value;
				} elsif ( $element->isType(EDG::WP4::CCM::Element::LIST) ) {
					while ( $element->hasNextElement() ) {
						my $elem = $element->getNextElement();
						if ( $elem->isProperty() ) {
							my $value = $elem->getValue();
							push ( @{ $giisallow{$no}{$entry} }, $value);
						}	
					}	
				}	
			}	
		}
	}	
	
	my %giisreg;
	if ($config->elementExists('/software/components/globuscfg/mds/giis/registration')) {
		my $mgre = 
			$config->getElement('/software/components/globuscfg/mds/giis/registration');
	    	while( $mgre->hasNextElement() ) {
			my $grelement = $mgre->getNextElement();
			my $name = $grelement->getName();
			while ( $grelement->hasNextElement() ) {
				my $element = $grelement->getNextElement();
				my $entry = $element->getName();
				my $value = $element->getValue();
				$giisreg{$name}{$entry} = $value;
			}	
		}	   
	}	

	# GRIDFtp

	my %gridftp;
	if ($config->elementExists('/software/components/globuscfg/gridftp/')) {
		my $fgfe = $config->getElement('/software/components/globuscfg/gridftp/');
		while ( $fgfe->hasNextElement() ) {
			my $element = $fgfe->getNextElement();
			my $name = $element->getName();
			my $value = $element->getValue();
			$gridftp{$name} = $value;
		}				 
	}

	# [gatekeeper]
	my %gatekeeper;
	if ($config->elementExists('/software/components/globuscfg/gatekeeper/')) {
		my $ggfe = $config->getElement('/software/components/globuscfg/gatekeeper/');
		while ( $ggfe->hasNextElement() ) {
			my $element = $ggfe->getNextElement();
			my $name = $element->getName();
			next if $name =~ /^jobmanagers$/;
			if ( $element->isProperty() ) {
				my $value = $element->getValue();
				$gatekeeper{$name} = $value;
			} elsif ( $element->isType(EDG::WP4::CCM::Element::LIST) ) {
				while ( $element->hasNextElement() ) {
					my $elem = $element->getNextElement();
					my $value = $element->getValue();
					push( @{ $gatekeeper{$name} }, $value);
				}	
			}
		}
	}	
	
	# gatekeeper->jobmanagers
	if ($config->elementExists('/software/components/globuscfg/gatekeeper/jobmanagers')) {
		my $ggjfe = 
			$config->getElement('/software/components/globuscfg/gatekeeper/jobmanagers');
		while ( $ggjfe->hasNextElement() ) {
			my $jobmanager = $ggjfe->getNextElement();
			my $jobmanno = $jobmanager->getName();
			while ( $jobmanager->hasNextElement() ) {
				my $property = $jobmanager->getNextElement();
				my $name = $property->getName();
				my $value = $property->getValue();
				$gatekeeper{'jobmanagers'}{$jobmanno}{$name} = $value;
			}
		}
	}	

	# Generating the config file

	$contents = "";
	
	# [common]
	$contents .= "[common]\n";
	$contents .= "GLOBUS_LOCATION=$global{'GLOBUS_LOCATION'}\n";
	$contents .= "globus_flavor_name=$global{'globus_flavor_name'}\n";
	$contents .= "x509_user_cert=$global{'x509_user_cert'}\n" 
													if $global{'x509_user_cert'};
	$contents .= "x509_user_key=$global{'x509_user_key'}\n" if $global{'x509_user_key'};
	$contents .= "gridmap=$global{'gridmap'}\n" if $global{'gridmap'};
	$contents .= "gridmapdir=$global{'gridmapdir'}\n" if $global{'gridmapdir'};
	$contents .= "\n";

	# [mds]
	$contents .= "[mds]\n";
	$contents .= "globus_flavor_name=$mds{'globus_flavor_name'}\n" 
													if $mds{'globus_flavor_name'};
	$contents .= "user=$mds{'user'}\n" if $mds{'user'};
	$contents .= "\n";

	$contents .= "x509_user_cert=$mds{'x509_user_cert'}\n\n" if $mds{'x509_user_cert'};

	$contents .= "x509_user_key=$mds{'x509_user_key'}\n\n" if $mds{'x509_user_key'};
	
	# [mds/gris]
	$contents .= "[mds/gris]\n" . "suffix=$gris{'suffix'}\n\n" if $gris{'suffix'};
	
	# [mds/gris/provider/*]
	my $prov;
	foreach $prov ( keys %{ $gris{'provider'} } ) {
		$contents .= "[mds/gris/provider/$prov]\n";	
		$contents .= "provider=$gris{'provider'}{$prov}\n" if $gris{'provider'}{$prov};
		$contents .= "\n";
	}

	# [mds/gris/registration/*]
	my $regno;
	foreach $regno ( keys %{ $gris{'registration'} } ) {
		$contents .= "[mds/gris/registration/".
					 "$gris{'registration'}{$regno}{'recordname'}]\n";	
		$contents .= "regname=$gris{'registration'}{$regno}{'regname'}\n" 
					if $gris{'registration'}{$regno}{'regname'};
		$contents .= "reghn=$gris{'registration'}{$regno}{'reghn'}\n" 
					if $gris{'registration'}{$regno}{'reghn'};
		$contents .= "regport=$gris{'registration'}{$regno}{'regport'}\n" 
					if $gris{'registration'}{$regno}{'regport'};
		$contents .= "regperiod=$gris{'registration'}{$regno}{'regperiod'}\n" 
					if $gris{'registration'}{$regno}{'regperiod'};
		$contents .= "ttl=$gris{'registration'}{$regno}{'ttl'}\n" 
					if $gris{'registration'}{$regno}{'ttl'};
		$contents .= "\n";
	}

	# Structure for that was not literally taken from template
	# [mds/giis/*]
	my $allowno;
	foreach $allowno ( keys %giisallow ) {
		$contents .= "[mds/giis/$giisallow{$allowno}{'recordname'}]\n";	
		if ( $giisallow{$allowno}{'name'} ) {
			$contents .= "name=$giisallow{$allowno}{'name'}\n";
		} else {
			$contents .= "name=$giisallow{$allowno}{'recordname'}\n";
		}	
		if ( @{ $giisallow{$allowno}{'allowreg'} } ) {
			my $perm;
			foreach ( @{ $giisallow{$allowno}{'allowreg'} } ) {
				$contents .= "allowreg=\"$_\"\n";
			}
		}
		$contents .= "\n";
	}
	
	# Structure for that was not literally taken from template
	# [mds/giis/*]
	my $reg;
	foreach $reg ( keys %giisreg ) {
		if ( $giisreg{$reg}{'regname'} && $giisreg{$reg}{'reghn'} ) {
			$contents .= "[mds/giis/$reg/registration/$giisreg{$reg}{'regname'}]\n";	
			if ( $giisreg{$reg}{'name'} ) {
				$contents .= "name=$giisreg{$reg}{'name'}\n";
			} else {
				$contents .= "name=$reg\n";
			}	
			$contents .= "regname=$giisreg{$reg}{'regname'}\n" ;
			$contents .= "reghn=$giisreg{$reg}{'reghn'}\n";
			$contents .= "regport=$giisreg{$reg}{'regport'}\n" 
													if $giisreg{$reg}{'regport'};
			$contents .= "regperiod=$giisreg{$reg}{'regperiod'}\n" 
													if $giisreg{$reg}{'regperiod'};
			$contents .= "ttl=$giisreg{$reg}{'ttl'}\n" if $giisreg{$reg}{'ttl'};
			$contents .= "\n";
		}
	}

	# [gridftp]
	$contents .= "[gridftp]\n";
	$contents .= "globus_flavor_name=$gridftp{'globus_flavor_name'}\n" 
													if $gridftp{'globus_flavor_name'};
	$contents .= "X509_USER_CERT=$gridftp{'X509_USER_CERT'}\n" 
													if $gridftp{'X509_USER_CERT'};
	$contents .= "X509_USER_KEY=$gridftp{'X509_USER_KEY'}\n" if $gridftp{'X509_USER_KEY'};
	$contents .= "ftpd=$gridftp{'ftpd'}\n" if $gridftp{'ftpd'};
	$contents .= "port=$gridftp{'port'}\n" if $gridftp{'port'};
	$contents .= "umask=$gridftp{'umask'}\n" if $gridftp{'umask'};
	$contents .= "log=$gridftp{'log'}\n" if $gridftp{'log'};
	$contents .= "user=$gridftp{'user'}\n" if $gridftp{'user'};
	$contents .= "\n";

	# [gatekeeper]
	$contents .= "[gatekeeper]\n";
	$contents .= "globus_flavor_name=$gatekeeper{'globus_flavor_name'}\n" 
													if $gatekeeper{'globus_flavor_name'};
	$contents .= "default_jobmanager=". $gatekeeper{'jobmanagers'}{0}{'recordname'} . "\n"		if (defined($gatekeeper{'jobmanagers'}{0}));
	if ( $gatekeeper{'job_manager_path'} ) {
		my @job_manager_path = split ( /s+/,  ); 
		my $jmp;
		foreach $jmp (@{ $gatekeeper{'job_manager_path'} } ) {
			$contents .= "job_manager_path=\$GLOBUS_LOCATION/libexec/$jmp\n";
		}	
	} else {
			$contents .= "job_manager_path=\$GLOBUS_LOCATION/libexec/\n";
	}	
	$contents .= "extra_options=$gatekeeper{'extra_options'}\n" 
													if $gatekeeper{'extra_options'};
	$contents .= "globus_gatekeeper=$gatekeeper{'globus_gatekeeper'}\n" if $gatekeeper{'globus_gatekeeper'};
	$contents .= "user=$gatekeeper{'user'}\n" if $gatekeeper{'user'};
	$contents .= "port=$gatekeeper{'port'}\n" if $gatekeeper{'port'};
	$contents .= "logfile=$gatekeeper{'logfile'}\n" if $gatekeeper{'logfile'};
	$contents .= "\n";

#	# [gatekeeper/fork]
#	$contents .= "[gatekeeper/fork]\n";
#	$contents .= "type=fork\n";
#	$contents .= "job_manager=globus-job-manager\n";
#	$contents .= "\n";
	
	# [gatekeeper/*]
	my $jobmanno;
	if ( keys %{ $gatekeeper{'jobmanagers'} } ) {	
		$contents .= "jobmanagers=\"";
 		foreach $jobmanno (keys %{ $gatekeeper{'jobmanagers'} } ) {
			$contents .= $gatekeeper{'jobmanagers'}{$jobmanno}{'recordname'}. " ";
		}	
		$contents .= "\"\n\n";
	}	
 	foreach $jobmanno (keys %{ $gatekeeper{'jobmanagers'} } ) {
 		$contents .= "[gatekeeper/$gatekeeper{'jobmanagers'}{$jobmanno}{'recordname'}]\n";
 		if ( $gatekeeper{'jobmanagers'}{$jobmanno}{'type'} ) {
 			$contents .= "type=$gatekeeper{'jobmanagers'}{$jobmanno}{'type'}\n"; 
 		} else {
 			$contents .= "type=$gatekeeper{'jobmanagers'}{$jobmanno}{'recordname'}\n"; 
 		}	
 		$contents .= 	
 			"job_manager=$gatekeeper{'jobmanagers'}{$jobmanno}{'job_manager'}\n" 
 							if $gatekeeper{'jobmanagers'}{$jobmanno}{'job_manager'};
 		$contents .=
 			"extra_config=$gatekeeper{'jobmanagers'}{$jobmanno}{'extra_config'}\n" 
 							if $gatekeeper{'jobmanagers'}{$jobmanno}{'extra_config'};
 		$contents .= "\n";
 	}
							
	if ( ! -e $global{'GLOBUS_CONFIG'} ) {
	  	open ( CONF,"> $global{'GLOBUS_CONFIG'}" );
		print CONF $contents;
		close ( CONF );
		$self->log("System configuration file $global{'GLOBUS_CONFIG'} created\n");
	} else {
		my $result = LC::Check::file( $global{'GLOBUS_CONFIG'},
				backup => ".old",
				contents => $contents
			);
		if ( $result ) {
			$self->log("System configuration file $global{'GLOBUS_CONFIG'} updated\n");
		}
	}		



	# Components execute (at boot time) with the system default path and
	# not the one built up from the profile.d scripts.  Consequently,
	# the locations of the batch system binaries may need to be added to
	# the path.

	if ($config->elementExists('/software/components/globuscfg/paths')) {
		my $pathe = 
			$config->getElement('/software/components/globuscfg/paths');
		my @newpaths;	
    	while( $pathe->hasNextElement() ) {
			my $element = $pathe->getNextElement();
			my $value = $element->getValue();
			push ( @newpaths, $value );
		}

		# Split the current path.
		my @pathelements = split(':',$ENV{PATH});
		
		# Get "paths" if it exists and add paths to array.
		foreach (@newpaths) {
			push @pathelements, $_;
		}

		# Make new path and setup the environment. 
		my $newpath = join(':',@pathelements);
		$ENV{PATH} = $newpath;
	}

	# Before starting anything, the globus initialization script must be run.
	# Do this after the sysconfig file has been setup, as the globus 
	# location is taken from there. 
	$ENV{'GLOBUS_LOCATION'} = $global{'GLOBUS_LOCATION'};
	my $output = `\$GLOBUS_LOCATION/sbin/globus-initialization.sh 2>&1`;
	$self->verbose("$output");
	if ( $? ) {
		$self->error("Unable to initialize globus. Return value: ". $? .
				" Output: " .$output);
		return;
	}	



	# Stop the daemons with the old files in place.  We'll stop
	# the wrong daemon if the executable has changed
	# in the configuration (e.g. gatekeeper)


	if ($config->elementExists('/software/components/globuscfg/services')) {
		my $serve = 
			$config->getElement('/software/components/globuscfg/services');
		my @services;	
    	while( $serve->hasNextElement() ) {
			my $element = $serve->getNextElement();
			my $value = $element->getValue();
			push ( @services, $value );
		}
		my $service;
		foreach $service (@services) {
			# Only stop the daemon if it is already running.  This depends
			# on the status directive returning a reasonable value (i.e.
			# 0 if running).
			if (! system("$sbinserv $service status")) {
				if (system("$sbinserv $service stop")) {
					$self->warn("init.d $service stop failed: ". $?);
				}
			}
		}
		# Start the daemons.
		foreach $service (@services) {
			if (system("$sbinserv $service start")) {
				$self->error("init.d $service start failed: ". $?);
			}
		}

	}	

	return 1; #OK
}

1 # Required for Perl modules
